@@ -75,7 +75,7 @@ def efetch_IPGs(ids, output_handle=None):
 
         if response.status_code != 200:
             raise requests.HTTPError(
-                "Error fetching sequences from NCBI [code {response.status_code}]."
+                f"Error fetching sequences from NCBI [code {response.status_code}]."
             )
 
         table += response.text
@@ -220,6 +220,7 @@ def parse_IPG_table(results, hits):
             # Copy the original Hit object and add contextual information
             subject = Subject(
                 hits=[hit.copy(subject=entry.protein_id) for hit in hit_list],
+                name=entry.protein_id,
                 ipg=ipg,
                 end=int(entry.end),
                 start=int(entry.start),
@@ -255,78 +256,38 @@ def find_identifier(qualifiers):
     return None
 
 
-def query_local_DB(hits, database):
-    """Build Organisms/Scaffolds using database.DB instance.
-
-    This function essentially mirrors parse_IPG_table, but is adapted to the JSON
-    database created using cblaster makedb. Protein headers in the DIAMOND database
-    follow the form "i_j_k" where i, j and k refer to the database indexes of organisms,
-    scaffolds and proteins, respectively. For example, >2_56_123 refers to the 123rd
-    protein of the 56th scaffold of the 2nd organism in the database. Context of each
-    hit is found by directly accessing those indices in the database, and then
-    Organism, Scaffold and Subject objects are generated as in parse_IPG_table.
-
-    Args:
-        hits (list): Hit objects created during cblaster search.
-        database (database.DB): cblaster database object.
-    Returns:
-        Organism objects containing hits sorted into genomic scaffolds.
+def query_local_DB(hits, db):
+    """Queries a local SQLite3 database created using the makedb module.
     """
-
     organisms = defaultdict(dict)
-
-    # Form non-redundant dictionary of hits. Each key will become a unique Subject.
     hit_dict = defaultdict(list)
     for hit in hits:
         hit_dict[hit.subject].append(hit)
-
-    for hit_index, hits in hit_dict.items():
-        # Hit headers should follow form "i_j_k", where i, j and k refer to the
-        # database indexes of organisms, scaffolds and proteins, respectively.
-        # e.g. >2_56_123 => 123rd protein of 56th scaffold of the 2nd organism
-        try:
-            i, j, k = [int(index) for index in hit_index.split("_")]
-        except ValueError:
-            LOG.exception("Hit has malformed header")
-
-        organism = database.organisms[i]
-        scaffold = organism.scaffolds[j]
-        protein = scaffold.features[k]
-
-        # For brevity...
-        org = organism.name
-        st = organism.strain
-        sc = scaffold.accession
-
-        # Instantiate new Organism/Scaffold objects on first encounter
-        if st not in organisms[org]:
-            organisms[org][st] = Organism(org, st)
-        if sc not in organisms[org][st].scaffolds:
-            organisms[org][st].scaffolds[sc] = Scaffold(sc)
-
-        # Want to report just protein ID, not lineage
-        identifier = find_identifier(protein.qualifiers)
-        if not identifier:
-            LOG.warning("Could not find identifier for hit %, skipping", hit.subject)
-            continue
-        for hit in hits:
-            hit.subject = identifier
-
-        # Save genomic location on the Hit instance
-        subject = Subject(
-            hits=hits,
-            start=protein.location.min(),
-            end=protein.location.max(),
-            strand=protein.location.strand
-        )
-
-        organisms[org][st].scaffolds[sc].subjects.append(subject)
-
-    return [
+    for (
+        rowid,
+        name,
+        start_pos,
+        end_pos,
+        strand,
+        scaffold,
         organism
-        for strains in organisms.values()
-        for organism in strains.values()
-    ]
+    ) in database.query_database(list(hit_dict), db):
+        if organism not in organisms:
+            organisms[organism] = Organism(organism, "")
+        if scaffold not in organisms[organism].scaffolds:
+            organisms[organism].scaffolds[scaffold] = Scaffold(scaffold)
+        hits = hit_dict[str(rowid)]
+        for hit in hits:
+            hit.subject = name
+        subject = Subject(
+            name=name,
+            hits=hits,
+            start=int(start_pos),
+            end=int(end_pos),
+            strand="+" if strand == 1 else "-"
+        )
+        organisms[organism].scaffolds[scaffold].subjects.append(subject)
+    return [organism for organism in organisms.values()]
 
 
 def cluster_satisfies_conditions(cluster, require=None, unique=3, minimum=3):
@@ -431,19 +392,19 @@ def find_clusters_in_organism(
     min_hits=3,
     gap=20000,
     require=None,
-    remote=True
+    remote=True,
+    query_sequence_order=None
 ):
     """Runs find_clusters() on all scaffolds in an organism."""
     for scaffold in organism.scaffolds.values():
-        scaffold.clusters = list(
-            find_clusters(
-                scaffold.subjects,
-                unique=unique,
-                min_hits=min_hits,
-                gap=gap,
-                require=require,
-            )
+        clusters = find_clusters(
+            scaffold.subjects,
+            unique=unique,
+            min_hits=min_hits,
+            gap=gap,
+            require=require,
         )
+        scaffold.add_clusters(clusters, query_sequence_order=query_sequence_order)
         LOG.debug(
             "Organism: %s, Scaffold: %s, Clusters: %i",
             organism.full_name,
@@ -480,15 +441,15 @@ def filter_session(
                         and hit.evalue < max_evalue
                     )
                 ]
-            scaffold.clusters = list(
-                find_clusters(
-                    scaffold.subjects,
-                    gap=gap,
-                    min_hits=min_hits,
-                    require=require,
-                    unique=unique,
-                )
+            clusters = find_clusters(
+                scaffold.subjects,
+                gap=gap,
+                min_hits=min_hits,
+                require=require,
+                unique=unique,
             )
+            scaffold.clusters = []
+            scaffold.add_clusters(clusters, query_sequence_order=session.queries)
         deduplicate(organism)
 
 
@@ -499,7 +460,7 @@ def calculate_gne(session):
     Median gn size
     """
     clusters = [
-        cluster[-1].end - cluster[0].start
+        cluster.end - cluster.start
         for organism in session.organisms
         for accession, scaffold in organism.scaffolds.items()
         for cluster in scaffold.clusters
@@ -513,7 +474,7 @@ def calculate_gne(session):
     )
 
 
-def estimate_neighbourhood(session, max_gap=1000000, samples=1000, scale="log"):
+def estimate_neighbourhood(session, max_gap=100000, samples=100, scale="linear"):
     """Estimate gene neighbourhood of a cblaster session."""
     if scale == "linear":
         space = np.linspace(0, max_gap, num=samples)
@@ -538,11 +499,12 @@ def estimate_neighbourhood(session, max_gap=1000000, samples=1000, scale="log"):
 
 def search(
     hits,
+    sqlite_db=None,
     unique=3,
     min_hits=3,
     gap=20000,
     require=None,
-    json_db=None,
+    query_sequence_order=None,
     ipg_file=None
 ):
     """Gets the genomic context for a collection of Hit objects.
@@ -556,7 +518,6 @@ def search(
     ...     min_hits=3,  # minimum number of hits in a cluster
     ...     gap=20000,  # maximum intergenic gap
     ...     require=["q1", "q2"],  # specific query sequences that MUST be in a cluster
-    ...     json_db=None,  # query a JSON database created using cblaster makedb
     ... )
 
     Args:
@@ -565,14 +526,13 @@ def search(
         unique (int): Unique query sequence threshold.
         min_hits (int): Minimum number of hits in a hit cluster.
         gap (int): Maximum intergenic distance (bp) between any two hits in a cluster.
-        json_db (str): Path to a JSON database created with cblaster makedb.
+        query_sequence_order (list): list of sequences of the order in the query file, is
     Returns:
         Dictionary of Organism objects keyed on species name.
     """
-    if json_db:
-        LOG.info("Loading JSON database: %s", json_db)
-        db = database.Database.from_json(json_db)
-        organisms = query_local_DB(hits, db)
+    if sqlite_db:
+        LOG.info("Querying local SQLite3 database: %s", sqlite_db)
+        organisms = query_local_DB(hits, sqlite_db)
     else:
         rows = efetch_IPGs(
             [hit.subject for hit in hits],
@@ -588,7 +548,8 @@ def search(
             min_hits=min_hits,
             gap=gap,
             require=require,
-            remote=json_db is None,
+            query_sequence_order=query_sequence_order,
+            remote=sqlite_db is None,
         )
 
     return organisms
