@@ -6,6 +6,7 @@ import sys
 
 from pathlib import Path
 
+
 from cblaster import (
     context,
     database,
@@ -13,6 +14,7 @@ from cblaster import (
     local,
     remote,
     parsers,
+    extract,
 )
 from cblaster.classes import Session
 from cblaster.plot import plot_session, plot_gne
@@ -27,22 +29,6 @@ logging.basicConfig(
 LOG = logging.getLogger(__name__)
 
 
-def makedb(genbanks, filename, indent=None):
-    """Generate JSON and diamond databases."""
-    LOG.info("Starting cblaster makedb")
-    db = database.Database.from_files(genbanks)
-
-    LOG.info("Writing FASTA file with database sequences: %s", filename + ".faa")
-    LOG.info("Building DIAMOND database: %s", filename + ".dmnd")
-    db.makedb(filename)
-
-    LOG.info("Building JSON database: %s", filename + ".json")
-    with open(f"{filename}.json", "w") as handle:
-        db.to_json(handle, indent=indent)
-
-    LOG.info("Done.")
-
-
 def gne(
     session,
     output=None,
@@ -85,7 +71,6 @@ def cblaster(
     query_file=None,
     query_ids=None,
     mode=None,
-    json_db=None,
     database=None,
     gap=20000,
     unique=3,
@@ -113,6 +98,7 @@ def cblaster(
     blast_file=None,
     ipg_file=None,
     hitlist_size=None,
+    cpus=None,
 ):
     """Run cblaster.
 
@@ -122,7 +108,6 @@ def cblaster(
         query_file (str): Path to FASTA format query file
         query_ids (list): NCBI protein sequence identifiers
         mode (str): Search mode ('local' or 'remote')
-        json_db (str): JSON database created with cblaster makedb
         database (str): Search database (NCBI if remote, DIAMOND if local)
         gap (int): Maximum gap (kilobase) between cluster hits
         unique (int): Minimum number of query sequences with hits in clusters
@@ -173,7 +158,11 @@ def cblaster(
                     session.to_json(fp, indent=indent)
     else:
         session = Session(
-            query_ids if query_ids else [],
+            queries=query_ids if query_ids else [],
+            sequences=helpers.get_sequences(
+                query_file=query_file,
+                query_ids=query_ids,
+            ),
             params={
                 "mode": mode,
                 "database": database,
@@ -182,21 +171,22 @@ def cblaster(
                 "max_evalue": max_evalue,
             },
         )
-        if query_file:
-            with open(query_file) as fp:
-                sequences = helpers.parse_fasta(fp)
-            session.queries = list(sequences)
-            session.params["query_file"] = query_file
 
-        if json_db:
-            session.params["json_db"] = json_db
+        if query_file:
+            # get_sequences() returns OrderedDict, so save keys to
+            # preserve query order
+            session.queries = list(session.sequences)
+            session.params["query_file"] = query_file
 
         if mode == "local":
             LOG.info("Starting cblaster in local mode")
+            sqlite_db = Path(database).with_suffix(".sqlite3")
+            if not sqlite_db.exists():
+                LOG.error("Could not find matching SQlite3 database, exiting")
+                raise SystemExit
             results = local.search(
                 database,
-                query_file=query_file,
-                query_ids=query_ids,
+                sequences=session.sequences,
                 min_identity=min_identity,
                 min_coverage=min_coverage,
                 max_evalue=max_evalue,
@@ -204,13 +194,10 @@ def cblaster(
             )
         elif mode == "remote":
             LOG.info("Starting cblaster in remote mode")
-
             if entrez_query:
                 session.params["entrez_query"] = entrez_query
-
             rid, results = remote.search(
-                query_file=query_file,
-                query_ids=query_ids,
+                sequences=session.sequences,
                 rid=rid,
                 database=database,
                 min_identity=min_identity,
@@ -220,18 +207,24 @@ def cblaster(
                 blast_file=blast_file,
                 hitlist_size=hitlist_size,
             )
+            sqlite_db = None
             session.params["rid"] = rid
 
+        if sqlite_db:
+            session.params["sqlite_db"] = sqlite_db
+
         LOG.info("Found %i hits meeting score thresholds", len(results))
         LOG.info("Fetching genomic context of hits")
+
         session.organisms = context.search(
             results,
+            sqlite_db=sqlite_db,
             unique=unique,
             min_hits=min_hits,
             gap=gap,
             require=require,
-            json_db=json_db,
             ipg_file=ipg_file,
+            query_sequence_order=list(session.sequences)
         )
 
         if session_file:
@@ -278,14 +271,19 @@ def main():
         LOG.setLevel(logging.DEBUG)
 
     if args.subcommand == "makedb":
-        makedb(args.genbank, args.filename, args.indent)
+        database.makedb(
+            args.paths,
+            args.filename,
+            cpus=args.cpus,
+            batch=args.batch,
+            force=args.force,
+        )
 
     elif args.subcommand == "search":
         cblaster(
             query_file=args.query_file,
             query_ids=args.query_ids,
             mode=args.mode,
-            json_db=args.json_db,
             database=args.database,
             gap=args.gap,
             unique=args.unique,
@@ -313,6 +311,7 @@ def main():
             blast_file=args.blast_file,
             ipg_file=args.ipg_file,
             hitlist_size=args.hitlist_size,
+            cpus=args.cpus,
         )
 
     elif args.subcommand == "gui":
@@ -332,6 +331,18 @@ def main():
             plot=args.plot,
         )
 
+    elif args.subcommand == "extract":
+        extract.extract(
+            args.session,
+            download=args.download,
+            output=args.output,
+            queries=args.queries,
+            organisms=args.organisms,
+            scaffolds=args.scaffolds,
+            name_only=args.name_only,
+            delimiter=args.delimiter,
+        )
+
 
 if __name__ == "__main__":
     main()
