@@ -10,17 +10,38 @@ from cblaster import __version__
 def add_makedb_subparser(subparsers):
     makedb = subparsers.add_parser(
         "makedb",
-        help="Generate JSON/diamond databases from GenBank files"
+        help="Generate local databases from genome files"
     )
     makedb.add_argument(
-        "genbank",
-        help="Path/s to GenBank files to use when building JSON/diamond databases",
+        "paths",
+        help="Path/s to genome files to use when building local databases",
         nargs="+",
     )
     makedb.add_argument(
         "filename",
-        help="Name to use when building JSON/diamond databases (with extensions"
-        " .json and .dmnd, respectively)",
+        help="Name to use when building SQLite3/diamond databases (with extensions"
+        " .sqlite3 and .dmnd, respectively)",
+    )
+    makedb.add_argument(
+        "-c",
+        "--cpus",
+        type=int,
+        help="Number of CPUs to use when parsing genome files. By default, all"
+        " available cores will be used.",
+    )
+    makedb.add_argument(
+        "-b",
+        "--batch",
+        type=int,
+        help="Number of genome files to parse before saving them in the local"
+        " database. Useful when encountering memory issues with large/many"
+        " files. By default, all genome files will be parsed at once."
+    )
+    makedb.add_argument(
+        "-f",
+        "--force",
+        action="store_true",
+        help="Overwrite pre-existing files, if any"
     )
 
 
@@ -160,6 +181,13 @@ def add_searching_group(search):
         " DIAMOND database (if 'local' is passed to --mode) or a valid NCBI"
         " database name (def. nr)",
     )
+    group.add_argument(
+        "-c",
+        "--cpus",
+        type=int,
+        help="Number of CPUs to use in local search. By default, all"
+        " available cores will be used.",
+    )
     group.add_argument(
         "-jdb",
         "--json_db",
@@ -383,6 +411,47 @@ def add_gne_subparser(subparsers):
     add_gne_output_group(gne)
 
 
+def add_extract_subparser(subparsers):
+    parser = subparsers.add_parser(
+        "extract",
+        help="Extract hit sequences from session files",
+        description="Extract information from session files",
+        epilog="Example usage\n-------------\n"
+        "Extract names of sequences matching a specific query:\n"
+        "  $ cblaster extract session.json -q \"Query1\"\n\n"
+        "Extract, download from NCBI and write to file in FASTA format:\n"
+        "  $ cblaster extract session.json -q \"Query1\" -d -o output.fasta\n\n"
+        "Extract only from specific organisms (regular expressions):\n"
+        "  $ cblaster extract session.json -or \"Aspergillus.*\" \"Penicillium.*\"\n\n"
+        "Generate delimited table (CSV) of all hits in clusters:\n"
+        "  $ cblaster extract session.json -de \",\"\n\n"
+        "Cameron Gilchrist, 2020",
+        formatter_class=argparse.RawDescriptionHelpFormatter,
+    )
+    parser.add_argument("session", help="cblaster session file")
+
+    fil = parser.add_argument_group("Filters")
+    fil.add_argument("-q", "--queries", help="IDs of query sequences", nargs="+")
+    fil.add_argument("-or", "--organisms", help="Organism names", nargs="+")
+    fil.add_argument("-sc", "--scaffolds", help="Scaffold names/ranges", nargs="+")
+
+    out = parser.add_argument_group("Output")
+    out.add_argument("-o", "--output", help="Output file name")
+    out.add_argument(
+        "-d",
+        "--download",
+        help="Fetch sequences from NCBI and write in FASTA format",
+        action="store_true",
+    )
+    out.add_argument(
+        "-no",
+        "--name_only",
+        help="Do not save sequence descriptions (i.e. no genomic coordinates)",
+        action="store_true",
+    )
+    out.add_argument("-de", "--delimiter", help="Sequence description delimiter")
+
+
 def get_parser():
     parser = argparse.ArgumentParser(
         "cblaster",
@@ -407,6 +476,7 @@ def get_parser():
     add_makedb_subparser(subparsers)
     add_search_subparser(subparsers)
     add_gne_subparser(subparsers)
+    add_extract_subparser(subparsers)
     return parser
 
 
@@ -418,7 +488,7 @@ def parse_args(args):
         parser.print_help()
         raise SystemExit
 
-    if arguments.subcommand in ("gui", "makedb", "gne"):
+    if arguments.subcommand in ("gui", "makedb", "gne", "extract"):
         return arguments
 
     if arguments.mode == "remote":
